/**************************************************************
 *
 *  Program Title:
 *
 *  Program Name:  External_Converters.c	  Version: 0.80
 *
 *  Author:  BR	  					  Date:  March 26, 2008
 *
 *
 *  This module provides functions to control the ADS1110 and the DAC8501
 *  These devices are not placed on the WASP by default so they may not be
 *  available to you.  They may however be soldered onto the board if required.
 *
 * Copyright 2009, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS,
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS,
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

#include "board.h"
#include "External_Converters.h"
#include "ARM_Serial.h"
#include "Wasp_Arm_Utilities.h"

static volatile AT91PS_PMC pPMC = AT91C_BASE_PMC;// pointer to PMC data structure
static volatile AT91PS_PIO pPIO = AT91C_BASE_PIOA;// pointer to PIO data structure
static volatile AT91PS_SPI pSPI = AT91C_BASE_SPI;
static volatile AT91PS_TWI pTWI = AT91C_BASE_TWI;

/*
 *Error Function
 */
int error(int err)
{
 	return err;
}

/*
 *Sends a control byte to the ADS1110.
 *If the ADC is not connected or if there
 *is an issue interfacing, -1 will be returned
 */
int InitI2CADC(unsigned char control)
{

	int status;

	pPMC->PMC_PCER = (1<<AT91C_ID_TWI);// enable the TWI clock
	pPMC->PMC_PCER = (1<<AT91C_ID_PIOA);// enable the Parallel IO Controller clock


	pPIO->PIO_ASR = (SDA | SCL);// PIO Peripheral A Select for SDA and SCL lines
	pPIO->PIO_PDR = (SDA | SCL);// PIO Controller PIO Disable, enables peripheral control
	pPIO->PIO_MDER = (SDA | SCL);//SDA & SCL are set as multi drive

	//reset the I2C device
	pTWI->TWI_CR = AT91C_TWI_SWRST;

 	//setting the SCK to ~111k
    //AT91C_TWI_CLDIV =>(0xFF <<  0) // (TWI) Clock Low Divider
    //AT91C_TWI_CHDIV => (0xFF <<  8) // (TWI) Clock High Divider
    //AT91C_TWI_CKDIV => (0x7 << 16) // (TWI) Clock Divider

 	pTWI->TWI_CWGR = (3 << 16) | (29 <<  8) | (29 <<  0);

 	pTWI->TWI_IDR = 0XFFFFFFFF;//disable all interrupts

 	//Master enable
 	pTWI->TWI_CR = AT91C_TWI_MSEN;

 	//setting up the slaves address & that we are doing to write (MREAD=0)
 	pTWI->TWI_MMR = (AT91C_TWI_DADR & (ADC_ADDRESS<<16));

 	//the byte of data to send to the control register
 	pTWI->TWI_THR = control;
 	//pTWI->TWI_CR = AT91C_TWI_START | AT91C_TWI_STOP;
 	//wait until the data leaves the THR buffer
 	while(!(pTWI->TWI_SR & AT91C_TWI_TXRDY));

 	//wait until the data shifts out & ensure that the data was ACK'ed
 	do{
 		status = pTWI->TWI_SR;
 		if (status & AT91C_TWI_NACK)//if the data was not acknowledged
 		{
 			return error(-1);
 		}
 	}
 	while(!(status & AT91C_TWI_TXCOMP));

 	//Disable the I2C
 	 pTWI->TWI_CR = AT91C_TWI_MSDIS;

 	return 0;
}

/*
 *This function will read the ADS1110
 * Does not work correctly :-(
 */
int ReadI2CADC(void)
{
 	int temp=0;//will store the read value

 	//Master enable
 	pTWI->TWI_CR = AT91C_TWI_MSEN;

 	//setting up the slaves address & that we are doing to read (MREAD=1)
 	//pTWI->TWI_MMR = (AT91C_TWI_DADR & (ADC_ADDRESS<<16)) | AT91C_TWI_MREAD;
 	pTWI->TWI_MMR = (AT91C_TWI_DADR & (ADC_ADDRESS<<16)) | AT91C_TWI_MREAD;

 	//send a start condition
 	pTWI->TWI_CR = AT91C_TWI_START;


 	while(!(pTWI->TWI_SR & AT91C_TWI_RXRDY));

 	temp = pTWI->TWI_RHR<<8;//read in the upper 8 bits first

 	//send a stop after we read the next byte
 	pTWI->TWI_CR = AT91C_TWI_STOP;
 	while(!(pTWI->TWI_SR & AT91C_TWI_RXRDY));

 	temp += pTWI->TWI_RHR;//LSB added to the correct location

 	while(!(pTWI->TWI_SR & AT91C_TWI_TXCOMP));
 	//Disable the I2C
 	pTWI->TWI_CR = AT91C_TWI_MSDIS;

 	//Will have to sign extend
 	if (temp & (1<<15))
 	{
 		temp |= 0xFFFF0000;
 	}
	return temp;
}


/*
 *Set up the DAC8501
 */
void InitDAC(void)
{
	 // enable the SPI peripheral clock

	 pPMC->PMC_PCER = (1<<AT91C_ID_SPI);// enable the Serial Peripheral Interface clock
	 pPMC->PMC_PCER = (1<<AT91C_ID_PIOA);// enable the Parallel IO Controller clock

	 pPIO->PIO_ASR = (MOSI | SCK);// PIO Peripheral A Select for MOSI and SCK lines
	 pPIO->PIO_PDR = (MOSI | SCK);// PIO Controller PIO Disable, enables peripheral control

	 pPIO->PIO_PER = DCS;// PIO Enable Register - allow PIO to control the DCS pin
	 pPIO->PIO_OER = DCS;// PIO Output Enable Register - sets DCS as an output
	 pPIO->PIO_SODR = DCS;//Output a 1 to deselect the DAC

	 pPIO->PIO_PPUDR = SS;//Enable the pullup on the SPI select line.  This will ensure that the SPI controller will not generate a mode fault (a low would indicate that it was selected as a slave


	 pSPI->SPI_MR = AT91C_SPI_MSTR | AT91C_SPI_MODFDIS;//SPI Master, Disable Mode fault detection
	 pSPI->SPI_CSR[0] = AT91C_SPI_CPOL | AT91C_SPI_NCPHA | AT91C_SPI_BITS_8 | AT91C_SPI_SCBR;//SPI Mode 2, Baud = MCK/0xFF, Keep the chip select line asserted
	 pSPI->SPI_CR = AT91C_SPI_SPIEN;//Enable the SPI controller
}

/*
 *Disable the SPI port and place the port directions
 *back to how they where
 */
/*
void ReleaseDAC(void)
{
	 SPI_DDR = old_DDR;
	 SPCR = 0x00;//Disable the SPI
}
*/
/*
 *Will write the data to the DAC8501
 *
 *            Vref X data
 *   Vout =  -------------
 *              65,536
 */

void WriteDAC(unsigned int data)
{
	//select the DAC8501
	pPIO->PIO_CODR = DCS;//Output a 0 to select the DAC


	//if you need to modify PD (power-down modes)
	//modify this data
	//ie the data is X, X, X, X, X, X, PD1, PD0
	pSPI->SPI_TDR = 0x00;
	//Wait until the we casn send more data
	while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));
	//Send out the upper byte first
	pSPI->SPI_TDR = data>>8;
	//Wait until the we casn send more data
	while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));

	//Last bits of data to send
	pSPI->SPI_TDR = (unsigned char)data;

	while(!(pSPI->SPI_SR & AT91C_SPI_TXEMPTY));//wait until the SPI transfere is finished

	//deselect the DAC8501
	pPIO->PIO_SODR = DCS;//Output a 1 to deselect the DAC
}

void InitFlash(void)
{
	 // enable the SPI peripheral clock

	 pPMC->PMC_PCER = (1<<AT91C_ID_SPI);// enable the Serial Peripheral Interface clock
	 pPMC->PMC_PCER = (1<<AT91C_ID_PIOA);// enable the Parallel IO Controller clock

	 pPIO->PIO_ASR = (MOSI | SCK | SS);// PIO Peripheral A Select for MOSI and SCK lines
	 pPIO->PIO_PDR = (MOSI | SCK | SS);// PIO Controller PIO Disable, enables peripheral control

	 pSPI->SPI_MR = AT91C_SPI_MSTR | AT91C_SPI_MODFDIS;//SPI Master, Disable Mode fault detection

	 //mode 0 or 3
	 pSPI->SPI_CSR[0] = AT91C_SPI_CSAAT | AT91C_SPI_NCPHA | AT91C_SPI_BITS_8 | AT91C_SPI_SCBR;//SPI Mode 0, Baud = MCK/0xFF

	 pSPI->SPI_CR = AT91C_SPI_SPIEN;//Enable the SPI controller
}

void Flash(unsigned int data)
{

	char mytext[20];

	pSPI->SPI_TDR = 0x9F;
	//Wait until the we can send more data
	while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));
	//Send out the upper byte first
	pSPI->SPI_TDR = 0x9F;

	while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));


	Print_Int(pSPI->SPI_RDR, mytext, HEX);
	uart0_puts(mytext);
	uart0_puts("\n");


	//Wait until the we casn send more data
	while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));

	//Last bits of data to send
	pSPI->SPI_TDR = 0x9F;
	while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));


	Print_Int(pSPI->SPI_RDR, mytext, HEX);
	uart0_puts(mytext);
	uart0_puts("\n");
	while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));

		//Last bits of data to send
		pSPI->SPI_TDR = 0x9F;
		while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));


		Print_Int(pSPI->SPI_RDR, mytext, HEX);
		uart0_puts(mytext);
		uart0_puts("\n");

	while(!(pSPI->SPI_SR & AT91C_SPI_TXEMPTY));//wait until the SPI transfere is finished
}



